% December 2019
% Developed by TeSA and CNES (France, Toulouse)
% contact
%   corinne.mailhes@tesa.prd.fr
%   amandine.guillot@cnes.fr
%
%        MAIN CODE
% 
% Computation of SLA spectra using several methods:
%   1-Periodogram (Tuckey window)
%   2-ARWARP (see paper)
% Slope Estimation is done on the corresponding spectra through :
%   Linear regression
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% 
% Necessary functions (the comments in these functions may be in French)
%   generation_signal_bruit_simule_SLA 
%       generation_signal_simule_SLA
%   estimePenteBruit
%   calculARWARP 
%       warping
%   get_index_norm
 

clear all; close all;

% Computation on real or simulated SLA signals
simuSLA = input('Computation on real (0) or simulated (1) SLA signals? ');

% simuSLA = 1; % = 1 computation on simulated SLA signals
% simuSLA = 0; % = 0 computation on real SLA signals
%


if simuSLA == 1,

    % Change parameters if required
    % structure Simu en entree de generation signal

    Simu.Slope = 3; % slope

    Simu.NSamples = 3000; % number of samples per segment
    Simu.NSegments = 10;  % number of segments
    Simu.NIters = 1; % should be = 1, internal parameter
    Simu.Dist = 0.319; % sampling distance in km (Sentinel-3)
    Simu.repexdetrend = 0; % could be used to add a tendency
    Simu.FMin = 0.001*1/Simu.Dist; % in km-1, frequency corresponding to the beginning of the slope (f1 in the paper)
    % 0.001 corresponds to f1 in normalized frequency
    Simu.Fcross = 0.01*1/Simu.Dist; % in km-1, frequency corresponding to the beginning of the noise floor
    Simu.NoiseLevel = 0.003; % noise level 
    % values in Simu.FMin, Simu.Fcross and Simu.NoiseLevel have been chosen
    % based on real signals (Sentinel 3)
    
    disp('Simulated SLA parameters');
    disp([' Slope = ' num2str(Simu.Slope)]);
    disp([' Number of segments = ' num2str(Simu.NSegments)]);
    disp([' Number of samples of each segment = ' num2str(Simu.NSamples)]);
    nom = ['Simulated SLA (slope = ' num2str(Simu.Slope) ')'];

    Fe = 1/Simu.Dist; % sampling frequency

    % call generation signal
    [nbSeg,nbPts,vSLA,F_true,Sx_true] = generation_signal_bruit_simule_SLA(Simu);

else
    % SLA signal file
    [nom,chemin] = uigetfile('*.nc','SLA file name'); % 

    SLA = ncread([chemin nom],'Matrix');
    dist = ncread([chemin nom],'Step_N1'); % sampling frequency in km
        
    Fe = 1/dist; % sampling frequency
    nbPts = size(SLA,1); % number of samples per segment
    nbSeg = size(SLA,2); % number of segments
    N = nbSeg*nbPts; % total number of samples (necessary to reshape)

    % SLA matrix transformed in vector (to be coherent with simulated SLA
    % signals)
    vSLA = reshape(SLA,1,N);
end

%%%%%%%%%%%%%% ANALYSIS PARAMETERS %%%%%%%%%%%%%%%%%%
% Spectral Analysis by Periodogram, Tukey 10%, zero padding
fen = tukeywin(nbPts,0.1); % replace by the selected window: rectwin(nbPts); etc.
factzeropad = 4; % zero padding factor
nfftSLA = 2^nextpow2(nbPts)*factzeropad; % zero padding necessary to better estimate the slope
freqSLA = linspace(0,Fe/2,nfftSLA/2+1); % frequency vector in km-1
freqSLA_norm = linspace(0,0.5,nfftSLA/2+1); % normalised frequency vector

% Spectral Analysis by ARWARP (see IGARSS paper)
b = 0.9; % warping parameter - recommanded: 0.9
P = 5; % ARWARP model order - recommanded: 5

disp('Spectral analysis parameters');
disp(['Periodogram with Tukey (10%) window and zero-padding by a factor ' num2str(factzeropad)]);
disp(['ARWARP with warping parameter = ' num2str(b) ' and model order = ' num2str(P)]);

% parameters for noise floor estimation
freqplateaubruit_norm = [0.25   0.5]; % noise floor is estimated in this interval (normalised frequencies)

% parameters for slope estimation
% for linear regression
% Frequency interval chosen based on cumulated periodogram

% Fourier based PSD estimation
SpecTF = zeros(nbSeg,nfftSLA);
for k = 1:nbSeg, % Spectral estimation and slope estimation is done on each segment
    x = zeros(nfftSLA,1);
    x = vSLA((k-1)*nbPts +1 : (k-1)*nbPts + nbPts)';
    x = detrend(x); % 
    
    % Spectral Analysis by Periodogram
    SpecTF(k,:) = abs(fft(x.*fen,nfftSLA)).^2 ./nbPts; % periodogram, window: fen, zero-padding
    % Correction  : to estimate the noise floor
    % Sigma of the noise = sqrt (K/(2*dist)) K being the level of noise
    % floor directly read on the spectrum
    SpecTF(k,:) = SpecTF(k,:) ./ (Fe/2);
end
PeriodoMoy = mean(SpecTF); % to set the linear regression interval in kms (following figure)

figure('Name','Mean Periodogram');
h = axes; 
loglog(1./freqSLA_norm(2:end)/Fe,PeriodoMoy(2:nfftSLA/2+1)); 
xlabel('km');
title( ['Mean PSD (Welch) of ' nom ]);
set(h,'XDir','reverse');

disp('Choice of wavelength interval for slope estimation via linear regression');
disp('------------------------------------------------------------------------');
disp('Linear regression is done between');
fkminkm = input('          Wavelength 1 in km: ');
disp('and');
fkmaxkm = input('          Wavelength 2 in km: ');
v = axis; hold on;
loglog([fkminkm fkminkm],[v(3) v(4)],'g--');
loglog([fkmaxkm fkmaxkm],[v(3) v(4)],'g--');

% transcription in normalized frequencies
fkmin = 1 / fkminkm / Fe; % in normalized frequencies
fkmax = 1 / fkmaxkm / Fe;
freqpente_norm = sort([fkmin fkmax]);% disp(['Linear regression is done between ' num2str(1/(freqpente_norm(2)*Fe)) ' km and ' num2str(1/(freqpente_norm(1)*Fe)) ' km']);
[pentespectre_moy, sigmabruit_moy,Sth_moy] = estimePenteBruit(PeriodoMoy(1:nfftSLA/2+1),freqSLA_norm,freqpente_norm,freqplateaubruit_norm);
disp(['Slope estimated on mean periodogram by linear regression with zero-padding: ' num2str(pentespectre_moy)]);

loglog(1./freqSLA_norm(2:end)/Fe,Sth_moy(2:nfftSLA/2+1),'r'); % display the slope and the noise floor


% vector initialisations
SpecARWARP = zeros(nbSeg,nfftSLA/2+1);
alphaTFreg = zeros(nbSeg,1);
alphaARreg = zeros(nbSeg,1);
sigmabruit = zeros(nbSeg,1);


for k = 1:nbSeg, % Spectral estimation and slope estimation is done on each segment
    x = zeros(nfftSLA,1);
    x = vSLA((k-1)*nbPts +1 : (k-1)*nbPts + nbPts)';
    x = detrend(x); % 
    
    % Spectral Analysis by Periodogram (done above)   
    SpectreSLA = SpecTF(k,1:nfftSLA/2+1);
    % Slope and noise floor estimation by Linear Regression on each
    % periodogram (on each segment)
    [pentespectre_reglin, sigmabruit_reglin,Sth_reglin] = estimePenteBruit(SpectreSLA,freqSLA_norm,freqpente_norm,freqplateaubruit_norm);
    sigmabruit(k) = sigmabruit_reglin; 
    alphaTFreg(k) = pentespectre_reglin;

   
    % Spectral Analysis by ARWARP
    disp(['Processing segment number ' num2str(k) '/' num2str(nbSeg) ' ... ']);
    SpecARWARP(k,:) = calculARWARP(x,P,b,freqSLA_norm);
    % Correction  : to estimate the noise floor
    % Sigma of the noise = sqrt (K/(2*dist)) K being the level of noise
    % floor directly read on the spectrum
    SpecARWARP(k,:) = SpecARWARP(k,:) ./ (Fe/2);
    
   
    SpectreSLA = SpecARWARP(k,:);
    % Slope estimation by Linear Regression on ARWARP
    [pentespectre_reglinW, sigmabruit_reglinW,Sth_reglinW] = estimePenteBruit(SpectreSLA,freqSLA_norm,freqpente_norm,freqplateaubruit_norm);
    alphaARreg(k) = pentespectre_reglinW;
    % noise floor is very well estimated by periodogram, do not estimate by
    % ARWARP
end


%%%%%% SLOPE ESIMATION ON SUCCESSIVE SEGMENTS OF THE SLA FILE
figure('Name','Slope estimation along the SLA segments');
MK = 10;
plot(alphaTFreg,'b-d','MarkerSize',MK);hold on;
plot(alphaARreg,'r-*','MarkerSize',MK);
v = axis;
plot([v(1) v(2)], [pentespectre_moy pentespectre_moy],'g --','LineWidth',3);
Leg = legend(['Periodogram: Mean = ' num2str(mean(alphaTFreg),'%.2f') ' +/- ' num2str(2*std(alphaTFreg),'%.2f') ],...
    ['ARWARP(' num2str(P) '): Mean = ' num2str(mean(alphaARreg),'%.2f') ' +/- ' num2str(2*std(alphaARreg),'%.2f') ],...
    ['Welch: estimated slope = ' num2str(pentespectre_moy)]);
Leg.FontSize = 14;

xlabel('SLA segment number');
ylabel('Slope estimate');
title(['Slope estimation along the segments of ' nom]);


%%%%%% DISPLAY OF THE PSD OF THE SUCCESSIVE SEGMENTS OF THE SLA FILE
disp('Display of the PSD of');
disp('   0 - selected segments');
disp(['   1 - all segments (' num2str(nbSeg) ' figures)']);
rep = input('Your answer: ');

fk = sort([fkmin fkmax]);
if rep == 1,
    for numero = 1 : nbSeg,
        figure('Name',['PSD of segment number ' num2str(numero) ]);
        loglog(freqSLA,SpecTF(numero,1:nfftSLA/2+1),'b'); hold on;
        loglog(freqSLA,SpecARWARP(numero,1:nfftSLA/2+1),'r');
        xlabel('Wavenumber (cpkm)');ylabel('PSD (m2/cpkm)');
        v = axis;
        v(1) = freqSLA(2); v(2) = freqSLA(end); axis(v)
        loglog([fkmin*Fe fkmin*Fe],[v(3) v(4)],'g--');
        loglog([fkmax*Fe fkmax*Fe],[v(3) v(4)],'g--');
        
        deb = SpecARWARP(2);
        loglog(fk*Fe,[deb deb*(fk(2)/fk(1))^(-alphaARreg(numero))],'r','LineWidth',3);
        text(fk(2)*Fe, deb*(fk(2)/fk(1))^(-alphaARreg(numero)),['\alpha = ' num2str(alphaARreg(numero)) ]);
        deb = SpecTF(2);
        loglog(fk*Fe,[deb deb*(fk(2)/fk(1))^(-alphaTFreg(numero))],'b','LineWidth',3);
        text(fk(2)*Fe, deb*(fk(2)/fk(1))^(-alphaTFreg(numero)),['\alpha = ' num2str(alphaTFreg(numero)) ]);
        title(['SLA segment ' num2str(numero) ' of ' nom ]);
    end
    
else

    while rep ==0,
    numero = input('Segment number for which display of the PSD is waited: ');

    figure('Name',['PSD of segment number ' num2str(numero) ]);
    loglog(freqSLA,SpecTF(numero,1:nfftSLA/2+1),'b'); hold on;
    loglog(freqSLA,SpecARWARP(numero,1:nfftSLA/2+1),'r');
    xlabel('Wavenumber (cpkm)');ylabel('PSD (m2/cpkm)');
    v = axis;
    v(1) = freqSLA(2); v(2) = freqSLA(end); axis(v)
    loglog([fkmin*Fe fkmin*Fe],[v(3) v(4)],'g--');
    loglog([fkmax*Fe fkmax*Fe],[v(3) v(4)],'g--');
   
    deb = SpecARWARP(2);
    loglog(fk*Fe,[deb deb*(fk(2)/fk(1))^(-alphaARreg(numero))],'r','LineWidth',3);
    text(fk(2)*Fe, deb*(fk(2)/fk(1))^(-alphaARreg(numero)),['\alpha = ' num2str(alphaARreg(numero)) ]);
    deb = SpecTF(2);
    loglog(fk*Fe,[deb deb*(fk(2)/fk(1))^(-alphaTFreg(numero))],'b','LineWidth',3);
    text(fk(2)*Fe, deb*(fk(2)/fk(1))^(-alphaTFreg(numero)),['\alpha = ' num2str(alphaTFreg(numero)) ]);
    title(['SLA segment ' num2str(numero) ' of ' nom ]);

    rep = input('One more PSD display of another segment ? (yes = 0): ');

    end
end


